// *****************************************************************************************
// ExampleAnimator.as
// 
// Copyright (c) 2007 Ryan Taylor | http://www.boostworthy.com
// 
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
// *****************************************************************************************
// 
// +          +          +          +          +          +          +          +          +
// 
// *****************************************************************************************

// PACKAGE /////////////////////////////////////////////////////////////////////////////////

package
{
	// IMPORTS /////////////////////////////////////////////////////////////////////////////
	
	import flash.display.DisplayObject;
	
	import com.boostworthy.animation.management.AnimationManager;
	import com.boostworthy.animation.rendering.RenderMethod;
	
	import GenericRollOverAnimation;
	import GenericRollOutAnimation;
	
	// CLASS ///////////////////////////////////////////////////////////////////////////////
	
	/**
	 * The 'ExampleAnimator' class is an example of how an extension of the animation 
	 * manager can be created on a per-project basis; thus allowing an API for adding 
	 * custom animation types and templates.
	 */
	public class ExampleAnimator extends AnimationManager
	{
		// *********************************************************************************
		// CLASS DECLERATIONS
		// *********************************************************************************
		
		// *********************************************************************************
		// EVENT HANDLERS
		// *********************************************************************************
		
		/**
		 * Constructor.
		 */
		public function ExampleAnimator()
		{
		}
		
		// *********************************************************************************
		// API
		// *********************************************************************************
		
		// ANIMATION TYPES /////////////////////////////////////////////////////////////////
		
		/**
		 * Generic roll over animation template.
		 * 
		 * @param	objTarget		A reference to the object being animated.
		 * @param	uRenderMethod	The render method to use for rendering this object. If one is not specified, the 'DEFAULT_RENDER_METHOD' is used.
		 * 
		 * @see		com.boostworthy.animation.rendering.RenderMethod
		 * @see		com.boostworthy.animation.management.AnimationManager#DEFAULT_RENDER_METHOD
		 */
		public function genericRollOver(objTarget:DisplayObject, uRenderMethod:uint = DEFAULT_RENDER_METHOD):void
		{
			// Set the global stage reference if one doesn't already exist.
			setStageReference(objTarget);
			
			// Add the new animation object to the framebuffer.
			m_objFramebuffer.addBufferObject(new GenericRollOverAnimation(objTarget, uRenderMethod));
		}
		
		/**
		 * Generic roll out animation template.
		 * 
		 * @param	objTarget		A reference to the object being animated.
		 * @param	uRenderMethod	The render method to use for rendering this object. If one is not specified, the 'DEFAULT_RENDER_METHOD' is used.
		 * 
		 * @see		com.boostworthy.animation.rendering.RenderMethod
		 * @see		com.boostworthy.animation.management.AnimationManager#DEFAULT_RENDER_METHOD
		 */
		public function genericRollOut(objTarget:DisplayObject, uRenderMethod:uint = DEFAULT_RENDER_METHOD):void
		{
			// Set the global stage reference if one doesn't already exist.
			setStageReference(objTarget);
			
			// Add the new animation object to the framebuffer.
			m_objFramebuffer.addBufferObject(new GenericRollOutAnimation(objTarget, uRenderMethod));
		}
	}
}