// *****************************************************************************************
// ExampleButton.as
// 
// Copyright (c) 2007 Ryan Taylor | http://www.boostworthy.com
// 
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
// *****************************************************************************************
// 
// +          +          +          +          +          +          +          +          +
// 
// *****************************************************************************************

// PACKAGE /////////////////////////////////////////////////////////////////////////////////

package
{
	// IMPORTS /////////////////////////////////////////////////////////////////////////////
	
	import flash.display.Sprite;
	import flash.events.MouseEvent;
	
	import com.boostworthy.animation.rendering.RenderMethod;
	
	import ExampleAnimator;
	
	// CLASS ///////////////////////////////////////////////////////////////////////////////
	
	/**
	 * The 'ExampleButton' class demonstrates the use of animation templates.
	 */
	public class ExampleButton extends Sprite
	{
		// *********************************************************************************
		// CLASS DECLERATIONS
		// *********************************************************************************
		
		// CONSTANTS ///////////////////////////////////////////////////////////////////////
		
		/**
		 * Configures the color of the box sprite.
		 */
		protected const BOX_COLOR:uint    = 0x006666;
		
		/**
		 * Configures the width of the box sprite.
		 */
		protected const BOX_WIDTH:Number  = 100;
		
		/**
		 * Configures the height of the box sprite.
		 */
		protected const BOX_HEIGHT:Number = 30;
		
		// CLASS MEMBERS ///////////////////////////////////////////////////////////////////
		
		/**
		 * Class instance of the example animator for animating this object.
		 */
		protected static var c_objAnimator:ExampleAnimator = new ExampleAnimator();
		
		// MEMBERS /////////////////////////////////////////////////////////////////////////
		
		/**
		 * A reference to this object's box sprite.
		 */
		protected var m_spBox:Sprite;
		
		// *********************************************************************************
		// EVENT HANDLERS
		// *********************************************************************************
		
		/**
		 * Constructor.
		 */
		public function ExampleButton()
		{
			// Initialize this object.
			init();
		}
		
		/**
		 * Called when this object is rolled over by the mouse cursor.
		 * 
		 * @param	objEvent	An object containing information about the mouse event.
		 */
		protected function onRollOver(objEvent:MouseEvent):void
		{
			// Animate this object using the generic roll over animation.
			c_objAnimator.genericRollOver(this, RenderMethod.TIMER);
		}
		
		/**
		 * Called when this object is rolled off of by the mouse cursor.
		 * 
		 * @param	objEvent	An object containing information about the mouse event.
		 */
		protected function onRollOut(objEvent:MouseEvent):void
		{
			// Animate this object using the generic roll out animation.
			c_objAnimator.genericRollOut(this, RenderMethod.TIMER);
		}
		
		// *********************************************************************************
		// INTERNAL
		// *********************************************************************************
		
		// INIT ////////////////////////////////////////////////////////////////////////////
		
		/**
		 * Initializes this object.
		 */
		protected function init():void
		{
			// Add event listeners for the roll over and out events.
			addEventListener(MouseEvent.ROLL_OVER, onRollOver);
			addEventListener(MouseEvent.ROLL_OUT, onRollOut);
			
			// Set all default property values.
			setDefaultValues();
			
			// Create this object's graphics.
			createGraphics();
		}
		
		/**
		 * Applies all default values to the corresponding properties.
		 */
		protected function setDefaultValues():void
		{
			// Apply the default values.
			buttonMode    = true;
			useHandCursor = true;
		}
		
		// GRAPHICS ////////////////////////////////////////////////////////////////////////
		
		/**
		 * Creates a box sprite and adds it to this object's display container.
		 */
		protected function createGraphics():void
		{
			// Create a new sprite and draw a box using the configured box settings.
			m_spBox      = new Sprite();
			m_spBox.name = "m_spBox";
			m_spBox.graphics.beginFill(BOX_COLOR);
			m_spBox.graphics.drawRect(0, 0, BOX_WIDTH, BOX_HEIGHT);
			
			// Initialize the color of this object.
			c_objAnimator.initColor(this, BOX_COLOR);
			
			// Add the box to the display container.
			addChild(m_spBox);
		}
	}
}