// *****************************************************************************************
// BasicSequenceExample.as
// 
// Copyright (c) 2007 Ryan Taylor | http://www.boostworthy.com
// 
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
// *****************************************************************************************
// 
// +          +          +          +          +          +          +          +          +
// 
// *****************************************************************************************

// PACKAGE /////////////////////////////////////////////////////////////////////////////////

package
{
	// IMPORTS /////////////////////////////////////////////////////////////////////////////
	
	import flash.display.Sprite;
	import flash.display.StageAlign;
	import flash.display.StageScaleMode;
	import flash.events.Event;
	import flash.events.MouseEvent;
	
	import com.boostworthy.animation.easing.Transitions;
	import com.boostworthy.animation.rendering.RenderMethod;
	import com.boostworthy.animation.sequence.Timeline;
	import com.boostworthy.animation.sequence.tweens.Tween;
	import com.boostworthy.core.Global;
	import com.boostworthy.events.AnimationEvent;
	
	// SWF /////////////////////////////////////////////////////////////////////////////////
	
	// Note: If using the Flash IDE to compile, these settings will be ignored.
	[SWF(backgroundColor="#111111", frameRate="31", width="550", height="400")]
	
	// CLASS ///////////////////////////////////////////////////////////////////////////////
	
	/**
	 * This example file provides you with a very basic example of using the timeline 
	 * object to create a sequence of animations. Simply launch the corresponding SWF 
	 * file and click the mouse down to watch the animation play forwards, then release
	 * the mouse button to watch it play in reverse.
	 * <p>
	 * Instructions for use:
	 * <p>
	 * Flash IDE users - this class is already set as the document class in the 
	 * corresponding FLA file for you. Simply open the FLA file and make any 
	 * changes to this file to experiment.
	 * <p>
	 * Non-Flash IDE users - Treat this class as your 'Main' class when you compile.
	 * 
	 * @see	com.boostworthy.animation.sequence.Timeline
	 */
	public class BasicSequenceExample extends Sprite
	{
		// *********************************************************************************
		// CLASS DECLERATIONS
		// *********************************************************************************
		
		// CONSTANTS ///////////////////////////////////////////////////////////////////////
		
		/**
		 * Configures the color of the box sprite.
		 */
		protected const BOX_COLOR:uint    = 0x006666;
		
		/**
		 * Configures the width of the box sprite.
		 */
		protected const BOX_WIDTH:Number  = 60;
		
		/**
		 * Configures the height of the box sprite.
		 */
		protected const BOX_HEIGHT:Number = 60;
		
		// MEMBERS /////////////////////////////////////////////////////////////////////////
		
		/**
		 * Holds an instance of the timeline object.
		 */
		protected var m_objTimeline:Timeline;
		
		/**
		 * Reference to the box sprite which will be created for the purpose
		 * of having something to animate.
		 */
		protected var m_spBox:Sprite;
		
		// *********************************************************************************
		// EVENT HANDLERS
		// *********************************************************************************
		
		// CONSTRUCTOR /////////////////////////////////////////////////////////////////////
		
		/**
		 * Constructor.
		 */
		public function BasicSequenceExample()
		{
			// Initialize this object.
			init();
		}
		
		// MOUSE EVENTS ////////////////////////////////////////////////////////////////////
		
		/**
		 * Called when the mouse is clicked down.
		 * 
		 * @param	objEvent	An object containing information about the mouse event.
		 */
		protected function onMouseDown(objEvent:MouseEvent):void
		{
			// Play the timeline forwards.
			m_objTimeline.play();
		}
		
		/**
		 * Called when the mouse is clicked up.
		 * 
		 * @param	objEvent	An object containing information about the mouse event.
		 */
		protected function onMouseUp(objEvent:MouseEvent):void
		{
			// Play the timeline backwards.
			m_objTimeline.playReverse();
		}
		
		// ANIMATION EVENTS ////////////////////////////////////////////////////////////////
		
		/**
		 * Called when the timeline starts playing.
		 * 
		 * @param	objEvent	An object containing information about the animation event.
		 */
		protected function onTimelineStart(objEvent:AnimationEvent):void
		{
			// Output a message about the event.
			trace("Timeline -> START");
		}
		
		/**
		 * Called when the timeline changes frames.
		 * 
		 * @param	objEvent	An object containing information about the animation event.
		 */
		protected function onTimelineChange(objEvent:AnimationEvent):void
		{
			// Output a message about the event.
			trace("Timeline -> CHANGE");
		}
		
		/**
		 * Called when the timeline stops playing.
		 * 
		 * @param	objEvent	An object containing information about the animation event.
		 */
		protected function onTimelineStop(objEvent:AnimationEvent):void
		{
			// Output a message about the event.
			trace("Timeline -> STOP");
		}
		
		/**
		 * Called when the timeline reaches the last frame of the timeline, unless looping is enabled.
		 * 
		 * @param	objEvent	An object containing information about the animation event.
		 */
		protected function onTimelineFinish(objEvent:AnimationEvent):void
		{
			// Output a message about the event.
			trace("Timeline -> FINISH");
		}
		
		// *********************************************************************************
		// INTERNAL
		// *********************************************************************************
		
		// INIT ////////////////////////////////////////////////////////////////////////////
		
		/**
		 * Initializes this object.
		 */
		protected function init():void
		{
			// Set all default property values.
			setDefaultValues();
			
			// Store a global reference to the stage.
			// This is needed for objects to have reference to the enter
			// frame event.
			Global.stage = stage;
			
			// Create a new timeline object.
			m_objTimeline = new Timeline(RenderMethod.ENTER_FRAME);
			
			// Uncomment these to sample event usage.
			/*
			// Add event listeners to the timeline.
			m_objTimeline.addEventListener(AnimationEvent.START,  onTimelineStart);
			m_objTimeline.addEventListener(AnimationEvent.CHANGE, onTimelineChange);
			m_objTimeline.addEventListener(AnimationEvent.STOP,   onTimelineStop);
			m_objTimeline.addEventListener(AnimationEvent.FINISH, onTimelineFinish);
			*/
			
			// Create a box sprite to animate.
			createBox();
			
			// Create an animation for the box sprite.
			createAnimation();
			
			// Add event listeners for the 'mouse down' and 'mouse up' events.
			stage.addEventListener(MouseEvent.MOUSE_DOWN, onMouseDown);
			stage.addEventListener(MouseEvent.MOUSE_UP, onMouseUp);
		}
		
		/**
		 * Applies all default values to the corresponding properties.
		 */
		protected function setDefaultValues():void
		{
			// Apply the default values.
			stage.scaleMode = StageScaleMode.NO_SCALE;
			stage.align     = StageAlign.TOP_LEFT;
		}
		
		// GRAPHICS ////////////////////////////////////////////////////////////////////////
		
		/**
		 * Creates a box sprite to animate.
		 */
		protected function createBox():void
		{
			// Create a new sprite and draw a box using the configured box settings.
			m_spBox      = new Sprite();
			m_spBox.name = "m_spBox";
			m_spBox.graphics.beginFill(BOX_COLOR);
			m_spBox.graphics.drawRect(-BOX_WIDTH / 2, -BOX_HEIGHT / 2, BOX_WIDTH, BOX_HEIGHT);
			
			// Add the box to the display container.
			addChild(m_spBox);
			
			// Start the box off in the center of the screen.
			m_spBox.x = Math.floor(stage.stageWidth / 2);
			m_spBox.y = Math.floor(stage.stageHeight / 2);
		}
		
		// ANIMATION ///////////////////////////////////////////////////////////////////////
		
		/**
		 * Creates a sequence of animations for the box sprite using the timeline.
		 */
		protected function createAnimation():void
		{
			// Create and add each tween to the timeline.
			m_objTimeline.addTween(new Tween(m_spBox, "x",         500, 1,  15, Transitions.SINE_IN_AND_OUT));
			m_objTimeline.addTween(new Tween(m_spBox, "y",         100, 7,  22, Transitions.SINE_IN_AND_OUT));
			m_objTimeline.addTween(new Tween(m_spBox, "rotation", -180, 7,  22, Transitions.SINE_IN_AND_OUT));
			m_objTimeline.addTween(new Tween(m_spBox, "x",         275, 16, 30, Transitions.SINE_IN_AND_OUT));
			m_objTimeline.addTween(new Tween(m_spBox, "scaleX",    0.5, 16, 30, Transitions.SINE_IN_AND_OUT));
			m_objTimeline.addTween(new Tween(m_spBox, "scaleY",    0.5, 16, 30, Transitions.SINE_IN_AND_OUT));
		}
	}
}