// *****************************************************************************************
// ColorManagementExample.as
// 
// Copyright (c) 2007 Ryan Taylor | http://www.boostworthy.com
// 
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
// *****************************************************************************************
// 
// +          +          +          +          +          +          +          +          +
// 
// *****************************************************************************************

// PACKAGE /////////////////////////////////////////////////////////////////////////////////

package
{
	// IMPORTS /////////////////////////////////////////////////////////////////////////////
	
	import flash.display.Sprite;
	import flash.display.StageAlign;
	import flash.display.StageScaleMode;
	import flash.events.Event;
	import flash.events.MouseEvent;
	
	import com.boostworthy.animation.easing.Transitions;
	import com.boostworthy.animation.management.AnimationManager;
	import com.boostworthy.animation.rendering.RenderMethod;
	import com.boostworthy.core.EnhancedSprite;
	
	// SWF /////////////////////////////////////////////////////////////////////////////////
	
	// Note: If using the Flash IDE to compile, these settings will be ignored.
	[SWF(backgroundColor="#111111", frameRate="31", width="550", height="400")]
	
	// CLASS ///////////////////////////////////////////////////////////////////////////////
	
	/**
	 * This example file provides you with an example of using the animation manager to 
	 * animate various color properties of an enhanced sprite. The 'enhanced' objects 
	 * located in the 'core' package implement the 'Colorable_I' interface which provides 
	 * access to a stored color matrix. The color matrix is necessary for allowing various 
	 * color properties such as brightness, contrast, hue, and saturation to be modified. 
	 * Simply launch the corresponding SWF file and roll over the box with the mouse cursor 
	 * to sample the animations.
	 * <p>
	 * Instructions for use:
	 * <p>
	 * Flash IDE users - this class is already set as the document class in the 
	 * corresponding FLA file for you. Simply open the FLA file and make any 
	 * changes to this file to experiment.
	 * <p>
	 * Non-Flash IDE users - Treat this class as your 'Main' class when you compile.
	 */
	public class ColorManagementExample extends Sprite
	{
		// *********************************************************************************
		// CLASS DECLERATIONS
		// *********************************************************************************
		
		// CONSTANTS ///////////////////////////////////////////////////////////////////////
		
		/**
		 * Configures the color of the box.
		 */
		protected const BOX_COLOR:uint    = 0x006666;
		
		/**
		 * Configures the width of the box.
		 */
		protected const BOX_WIDTH:Number  = 60;
		
		/**
		 * Configures the height of the box.
		 */
		protected const BOX_HEIGHT:Number = 60;
		
		// MEMBERS /////////////////////////////////////////////////////////////////////////
		
		/**
		 * Holds an instance of the animation manager.
		 */
		protected var m_objAnimationManager:AnimationManager;
		
		/**
		 * Reference to the enhanced sprite which will be created for the purpose
		 * of having something to animate.
		 */
		protected var m_spBox:EnhancedSprite;
		
		// *********************************************************************************
		// EVENT HANDLERS
		// *********************************************************************************
		
		/**
		 * Constructor.
		 */
		public function ColorManagementExample()
		{
			// Initialize this object.
			init();
		}
		
		// MOUSE EVENTS ////////////////////////////////////////////////////////////////////
		
		/**
		 * Called when the box is rolled over by the mouse cursor.
		 * 
		 * @param	objEvent	An object containing information about the mouse event.
		 */
		protected function onBoxRollOver(objEvent:MouseEvent):void
		{
			// Animate the brightness of the enhanced sprite.
			// 
			// Also available for enhanced objects: contrast, hue, sauration.
			m_objAnimationManager.brightness(m_spBox, 80, 500, Transitions.CUBIC_OUT, RenderMethod.TIMER);
		}
		
		/**
		 * Called when the box is rolled out by the mouse cursor.
		 * 
		 * @param	objEvent	An object containing information about the mouse event.
		 */
		protected function onBoxRollOut(objEvent:MouseEvent):void
		{
			// Animate the brightness of the enhanced sprite.
			// 
			// Also available for enhanced objects: contrast, hue, sauration.
			m_objAnimationManager.brightness(m_spBox, 0, 500, Transitions.CUBIC_OUT, RenderMethod.TIMER);
		}
		
		// *********************************************************************************
		// INTERNAL
		// *********************************************************************************
		
		// INIT ////////////////////////////////////////////////////////////////////////////
		
		/**
		 * Initializes this object.
		 */
		protected function init():void
		{
			// Set all default property values.
			setDefaultValues();
			
			// Create a new animation manager.
			m_objAnimationManager = new AnimationManager();
			
			// Create a box to animate.
			createBox();
		}
		
		/**
		 * Applies all default values to the corresponding properties.
		 */
		protected function setDefaultValues():void
		{
			// Apply the default values.
			stage.scaleMode = StageScaleMode.NO_SCALE;
			stage.align     = StageAlign.TOP_LEFT;
		}
		
		// GRAPHICS ////////////////////////////////////////////////////////////////////////
		
		/**
		 * Creates an enhanced sprite and draws a box to animate.
		 */
		protected function createBox():void
		{
			// Create a new enhanced sprite and draw a box using the configured box settings.
			m_spBox      = new EnhancedSprite();
			m_spBox.name = "m_spBox";
			m_spBox.graphics.beginFill(BOX_COLOR);
			m_spBox.graphics.drawRect(-BOX_WIDTH / 2, -BOX_HEIGHT / 2, BOX_WIDTH, BOX_HEIGHT);
			
			// Add the box to the display container.
			addChild(m_spBox);
			
			// Start the box off in the center of the screen.
			m_spBox.x = Math.floor(stage.stageWidth / 2);
			m_spBox.y = Math.floor(stage.stageHeight / 2);
			
			// Add event listeners for the when the mouse rolls over and out of the box.
			m_spBox.addEventListener(MouseEvent.ROLL_OVER, onBoxRollOver);
			m_spBox.addEventListener(MouseEvent.ROLL_OUT,  onBoxRollOut);
		}
	}
}