// *****************************************************************************************
// SequenceActionsExample.as
// 
// Copyright (c) 2007 Ryan Taylor | http://www.boostworthy.com
// 
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
// *****************************************************************************************
// 
// +          +          +          +          +          +          +          +          +
// 
// *****************************************************************************************

// PACKAGE /////////////////////////////////////////////////////////////////////////////////

package
{
	// IMPORTS /////////////////////////////////////////////////////////////////////////////
	
	import flash.display.Sprite;
	import flash.display.StageAlign;
	import flash.display.StageScaleMode;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.filters.BitmapFilterQuality;
	
	import com.boostworthy.animation.easing.Transitions;
	import com.boostworthy.animation.management.AnimationManager;
	import com.boostworthy.animation.rendering.RenderMethod;
	import com.boostworthy.animation.sequence.Timeline;
	import com.boostworthy.animation.sequence.tweens.Action;
	import com.boostworthy.animation.sequence.tweens.Tween;
	import com.boostworthy.core.Global;
	
	// SWF /////////////////////////////////////////////////////////////////////////////////
	
	// Note: If using the Flash IDE to compile, these settings will be ignored.
	[SWF(backgroundColor="#111111", frameRate="31", width="550", height="400")]
	
	// CLASS ///////////////////////////////////////////////////////////////////////////////
	
	/**
	 * This example file provides you with an example of using the timeline object to 
	 * create a complex animation sequence using both tweens and actions. Actions allow 
	 * you to call functions specific to forward and/or reverse playback when a specified 
	 * frame is reached. Simply launch the corresponding SWF file and click the mouse down 
	 * to watch the animation play forwards, then release the mouse button to watch it 
	 * play in reverse.
	 * <p>
	 * Instructions for use:
	 * <p>
	 * Flash IDE users - this class is already set as the document class in the 
	 * corresponding FLA file for you. Simply open the FLA file and make any 
	 * changes to this file to experiment.
	 * <p>
	 * Non-Flash IDE users - Treat this class as your 'Main' class when you compile.
	 * 
	 * @see	com.boostworthy.animation.sequence.Timeline
	 */
	public class SequenceActionsExample extends Sprite
	{
		// *********************************************************************************
		// CLASS DECLERATIONS
		// *********************************************************************************
		
		// CONSTANTS ///////////////////////////////////////////////////////////////////////
		
		/**
		 * Configures the color of the box sprite.
		 */
		protected const BOX_COLOR:uint    = 0x006666;
		
		/**
		 * Configures the width of the box sprite.
		 */
		protected const BOX_WIDTH:Number  = 60;
		
		/**
		 * Configures the height of the box sprite.
		 */
		protected const BOX_HEIGHT:Number = 60;
		
		// MEMBERS /////////////////////////////////////////////////////////////////////////
		
		/**
		 * Holds an instance of the timeline object.
		 */
		protected var m_objTimeline:Timeline;
		
		/**
		 * Holds an instance of the animation manager.
		 */
		protected var m_objAnimationManager:AnimationManager;
		
		/**
		 * Reference to the box sprite which will be created for the purpose
		 * of having something to animate.
		 */
		protected var m_spBox:Sprite;
		
		// *********************************************************************************
		// EVENT HANDLERS
		// *********************************************************************************
		
		// CONSTRUCTOR /////////////////////////////////////////////////////////////////////
		
		/**
		 * Constructor.
		 */
		public function SequenceActionsExample()
		{
			// Initialize this object.
			init();
		}
		
		// MOUSE EVENTS ////////////////////////////////////////////////////////////////////
		
		/**
		 * Called when the mouse is clicked down.
		 * 
		 * @param	objEvent	An object containing information about the mouse event.
		 */
		protected function onMouseDown(objEvent:MouseEvent):void
		{
			// Play the timeline forwards.
			m_objTimeline.play();
		}
		
		/**
		 * Called when the mouse is clicked up.
		 * 
		 * @param	objEvent	An object containing information about the mouse event.
		 */
		protected function onMouseUp(objEvent:MouseEvent):void
		{
			// Play the timeline backwards.
			m_objTimeline.playReverse();
		}
		
		// *********************************************************************************
		// INTERNAL
		// *********************************************************************************
		
		// INIT ////////////////////////////////////////////////////////////////////////////
		
		/**
		 * Initializes this object.
		 */
		protected function init():void
		{
			// Set all default property values.
			setDefaultValues();
			
			// Store a global reference to the stage.
			// This is needed for objects to have reference to the enter
			// frame event.
			Global.stage = stage;
			
			// Create a new timeline object.
			m_objTimeline = new Timeline(RenderMethod.ENTER_FRAME);
			
			// Create a new animation manager.
			m_objAnimationManager = new AnimationManager();
			
			// Create a box sprite to animate.
			createBox();
			
			// Create an animation for the box sprite.
			createAnimation();
			
			// Add event listeners for the 'mouse down' and 'mouse up' events.
			stage.addEventListener(MouseEvent.MOUSE_DOWN, onMouseDown);
			stage.addEventListener(MouseEvent.MOUSE_UP, onMouseUp);
		}
		
		/**
		 * Applies all default values to the corresponding properties.
		 */
		protected function setDefaultValues():void
		{
			// Apply the default values.
			stage.scaleMode = StageScaleMode.NO_SCALE;
			stage.align     = StageAlign.TOP_LEFT;
		}
		
		// GRAPHICS ////////////////////////////////////////////////////////////////////////
		
		/**
		 * Creates a box sprite to animate.
		 */
		protected function createBox():void
		{
			// Create a new sprite and draw a box using the configured box settings.
			m_spBox      = new Sprite();
			m_spBox.name = "m_spBox";
			m_spBox.graphics.beginFill(BOX_COLOR);
			m_spBox.graphics.drawRect(-BOX_WIDTH / 2, -BOX_HEIGHT / 2, BOX_WIDTH, BOX_HEIGHT);
			
			// Initialize the color of the box sprite so that it can be animated.
			m_objAnimationManager.initColor(m_spBox, BOX_COLOR);
			
			// Add the box to the display container.
			addChild(m_spBox);
			
			// Start the box off in the center of the screen.
			m_spBox.x = Math.floor(stage.stageWidth / 2);
			m_spBox.y = Math.floor(stage.stageHeight / 2);
		}
		
		// ANIMATION ///////////////////////////////////////////////////////////////////////
		
		/**
		 * Creates a sequence of animations for the box sprite using the timeline.
		 */
		protected function createAnimation():void
		{
			// Create and add some tweens to the timeline.
			m_objTimeline.addTween(new Tween(m_spBox, "x",         500, 1,  15, Transitions.SINE_IN_AND_OUT));
			m_objTimeline.addTween(new Tween(m_spBox, "y",         100, 7,  22, Transitions.SINE_IN_AND_OUT));
			
			// Call the 'forwardExample' method while playing forwards, and 
			// 'reverseExample' method while playing backwards.
			m_objTimeline.addTween(new Action(forwardExample, [], reverseExample, [], 7));
			
			// Create and add some more tweens to the timeline.
			m_objTimeline.addTween(new Tween(m_spBox, "rotation", -180, 7,  22, Transitions.SINE_IN_AND_OUT));
			m_objTimeline.addTween(new Tween(m_spBox, "x",         275, 16, 30, Transitions.SINE_IN_AND_OUT));
			
			// Call the 'changeBoxColor' method while playing forwards, and 
			// 'changeBoxBlur' method while playing backwards. A parameter for the 
			// reverse action handler is passed to demonstrate the parameters array usage.
			m_objTimeline.addTween(new Action(changeBoxColor, [], changeBoxBlur, [BitmapFilterQuality.HIGH], 16));
			
			// Create and add some more tweens to the timeline.
			m_objTimeline.addTween(new Tween(m_spBox, "scaleX",    0.5, 16, 30, Transitions.SINE_IN_AND_OUT));
			m_objTimeline.addTween(new Tween(m_spBox, "scaleY",    0.5, 16, 30, Transitions.SINE_IN_AND_OUT));
		}
		
		/**
		 * Example of a forward action handler.
		 */
		public function forwardExample():void
		{
			// Trace out a message for the example.
			trace("Playing forwards...");
		}
		
		/**
		 * Example of a reverse action handler.
		 */
		public function reverseExample():void
		{
			// Trace out a message for the example.
			trace("Playing backwards...");
		}
		
		/**
		 * Randomly animates the box sprite's color to a new value.
		 */
		public function changeBoxColor():void
		{
			// Animate the color of the box sprite.
			m_objAnimationManager.color(m_spBox, Math.floor(Math.random() * 0xFFFFFF), 1000, Transitions.QUINT_OUT, RenderMethod.ENTER_FRAME);
		}
		
		/**
		 * Randomly animates the box sprite's blur to a new value.
		 * 
		 * @param	nBlurQuality	The quality of the blur filter.
		 */
		public function changeBoxBlur(nBlurQuality:Number):void
		{
			// Generate a random amount of blur to apply to the box.
			var uBlurAmount:uint = Math.floor(Math.random() * 32);
			
			// Animate the blur of the box sprite.
			m_objAnimationManager.blur(m_spBox, uBlurAmount, uBlurAmount, nBlurQuality, 500, Transitions.QUINT_IN_AND_OUT, RenderMethod.ENTER_FRAME);
		}
	}
}