// *****************************************************************************************
// TwoDemo.as
// 
// Copyright (c) 2007 Ryan Taylor | http://www.boostworthy.com
// 
// Permission is hereby granted, free of charge, to any person
// obtaining a copy of this software and associated documentation
// files (the "Software"), to deal in the Software without
// restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the
// Software is furnished to do so, subject to the following
// conditions:
//
// The above copyright notice and this permission notice shall be
// included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
// OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
// HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
// WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
// FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
// OTHER DEALINGS IN THE SOFTWARE.
// *****************************************************************************************
// 
// +          +          +          +          +          +          +          +          +
// 
// *****************************************************************************************

// PACKAGE /////////////////////////////////////////////////////////////////////////////////

package
{
	// IMPORTS /////////////////////////////////////////////////////////////////////////////
	
	import flash.display.Bitmap;
	import flash.display.Sprite;
	import flash.display.StageAlign;
	import flash.display.StageScaleMode;
	import flash.events.Event;
	
	import com.boostworthy.animation.management.AnimationManager;
	import com.boostworthy.animation.rendering.RenderMethod;
	
	// SWF /////////////////////////////////////////////////////////////////////////////////
	
	// Note: If using the Flash IDE to compile, these settings will be ignored.
	[SWF(backgroundColor="#0F0C0A", frameRate="31", width="491", height="300")]
	
	// CLASS ///////////////////////////////////////////////////////////////////////////////
	
	/**
	 * In spirit of the v2 release of this animation system, the 'two demo' creates 
	 * individual sprites from each pixel of the number '2' and animates them to create 
	 * a field of particles.
	 * <p>
	 * Please note that you will need to make minor changes to this script to publish 
	 * it from the Flash IDE. This is because of the fact that an image is being embedded 
	 * using the 'embed' metadata tag.
	 */
	public class TwoDemo extends Sprite
	{
		// *********************************************************************************
		// CLASS DECLERATIONS
		// *********************************************************************************
		
		// MEMBERS /////////////////////////////////////////////////////////////////////////
		
		/**
		 * Holds an instance of the animation manager.
		 */
		protected var m_objAnimationManager:AnimationManager;
		
		/**
		 * Embed the 'two' bitmap.
		 */
		[Embed(source="assets/two.png")]
		protected var Two:Class;
		
		/**
		 * A bitmap for loading the 'two' asset into.
		 */
		protected var m_bmpTwo:Bitmap;
		
		/**
		 * A container to act as the field for the pixel particles.
		 */
		protected var m_spContainer:Sprite;
		
		/**
		 * Stores information about the current row number.
		 */
		protected var m_nRow:Number;
		
		// *********************************************************************************
		// EVENT HANDLERS
		// *********************************************************************************
		
		// CONSTRUCTOR /////////////////////////////////////////////////////////////////////
		
		/**
		 * Constructor.
		 */
		public function TwoDemo()
		{
			// Initialize this object.
			init();
		}
		
		/**
		 * Called when the stage is resized.
		 * 
		 * @param	objEvent	An object containing information about the event.
		 */
		protected function onStageResize(objEvent:Event):void
		{
			// Update the display based on the new stage size.
			updateDisplay();
		}
		
		// *********************************************************************************
		// INTERNAL
		// *********************************************************************************
		
		// INIT ////////////////////////////////////////////////////////////////////////////
		
		/**
		 * Initializes this object.
		 */
		protected function init():void
		{
			// Set all default property values.
			setDefaultValues();
			
			// Add a listener for the stage's resize event.
			stage.addEventListener(Event.RESIZE, onStageResize);
			
			// Create a new animation manager.
			m_objAnimationManager = new AnimationManager();
			
			// Create the magical 'two'.
			createTwo();
		}
		
		/**
		 * Applies all default values to the corresponding properties.
		 */
		protected function setDefaultValues():void
		{
			// Apply the default values.
			stage.scaleMode = StageScaleMode.NO_SCALE;
			stage.align     = StageAlign.TOP_LEFT;
			m_nRow          = 0;
		}
		
		// DISPLAY /////////////////////////////////////////////////////////////////////////
		
		/**
		 * Updates objects based on the current stage size.
		 */
		protected function updateDisplay():void
		{
			// Recenter the particle field.
			m_spContainer.x = Math.floor(stage.stageWidth  / 2 - 30);
			m_spContainer.y = Math.floor(stage.stageHeight / 2 - 50);
		}
		
		// GRAPHICS ////////////////////////////////////////////////////////////////////////
		
		/**
		 * Creates the magical 'two'.
		 */
		protected function createTwo():void
		{
			// Create a container for storing the pixel particles in and 
			// add it to the main display container.
			m_spContainer = new Sprite();
			addChild(m_spContainer);
			
			// Update the display.
			updateDisplay();
			
			// Create a new 'two' bitmap.
			m_bmpTwo = new Two();
			
			// Being the pixel particle creation process.
			stage.addEventListener(Event.ENTER_FRAME, addPixelRow);
		}
		
		/**
		 * Adds a row of pixel particles to the particle field.
		 * 
		 * @param	objEvent	An object containing information about the event.
		 */
		protected function addPixelRow(objEvent:Event):void
		{
			// Check to see if all rows have been filled.
			if(m_nRow < m_bmpTwo.height)
			{
				// Loop through the current column and add each pixel.
				for(var nCol:Number = 0; nCol < m_bmpTwo.width; nCol++)
				{
					// Add a pixel for the current row and column.
					addPixel(m_nRow, nCol);
				}
				
				// Increment the row number.
				m_nRow++
			}
			else
			{
				// Since the creation process is complete, remove the enter frame listener.
				stage.removeEventListener(Event.ENTER_FRAME, addPixelRow);
			}
		}
		
		/**
		 * Adds a new pixel particle at the specified row and column number.
		 * 
		 * @param	nRow	The row number to add a pixel at.
		 * @param	nCol	The column number to add a pixel at.
		 */
		protected function addPixel(nRow:Number, nCol:Number):void
		{
			// Check for transparency before adding a pixel.
			if((m_bmpTwo.bitmapData.getPixel32(nCol, m_nRow) >> 24) & 0xFF)
			{	
				// Create a new pixel and position it based on the current row 
				// and column numbers.
				var spPixel:Sprite = createPixel();
				spPixel.x = nCol;
				spPixel.y = m_nRow;
								
				// Add the pixel particle to the particle field.
				m_spContainer.addChild(spPixel);
						
				// Animate the pixel particle.
				m_objAnimationManager.pulse(spPixel, "x", nCol, nCol + (-20 + Math.floor(Math.random() * 40)), Math.floor(1000 + Math.random() * 4000), RenderMethod.ENTER_FRAME);
				m_objAnimationManager.pulse(spPixel, "y", m_nRow, m_nRow + (-20 + Math.floor(Math.random() * 40)), Math.floor(1000 + Math.random() * 4000), RenderMethod.ENTER_FRAME);
			}
		}
		
		/**
		 * Creates a new pixel particle.
		 * 
		 * @return	A new pixel particle.
		 */
		protected function createPixel():Sprite
		{
			// Create a new sprite, fill a one pixel box with color, and return it.
			var spPixel:Sprite = new Sprite();
			spPixel.graphics.beginFill(0x006666);
			spPixel.graphics.drawRect(0, 0, 1, 1);
			return spPixel;
		}
	}
}